----------------------------------------------
-- GlyphKeeper.lua
-- Written by Jason Doyle
-- snippets from various sources
-- This addon allows creating profiles which store currently active Glyphs and Talents
-- and will tell you what to replace when you choose a different profile.
----------------------------------------------

----------------------------------------------
--  UI Functions
----------------------------------------------

function GlyphKeeper_UI_ShowToggle()
	if GlyphKeeper_Main_UI:IsVisible() then
		GlyphKeeper_Main_UI:SetMovable(false)
		GlyphKeeper_Main_UI:Hide()
	else
		GlyphKeeper_Main_UI:Show()
		GlyphKeeper_Main_UI:SetMovable(true)
	end
end

function GlyphKeeper_UI_ShowSub(mode)
	if mode == "SaveMode" then
		GlyphKeeper_SubLabel:SetText("Save :")
		GlyphKeeper_SubTextBox:Enable()
	elseif mode == "UpdateMode" then
		GlyphKeeper_SubLabel:SetText("Update :")
		GlyphKeeper_SubTextBox:Disable()		 
	else 
		GlyphKeeper_SubLabel:SetText("Delete :")
		GlyphKeeper_SubTextBox:Disable()
	end
	
	GlyphKeeper_UI_Sub:Show()
end

function GlyphKeeper_UI_Delete()
	local arraytemp1, arraytemp2, boolFound = GlyphKeeper_GetProfile(strCurrentProfile)

	if boolFound then
		GlyphKeeper_DeleteProfile(strCurrentProfile)
		UIDropDownMenu_SetText(GlyphKeeper_Profile_Select, " ")
		GlyphKeeper_UI_HideSub()
		strCurrentProfile = nil
	end
end

function GlyphKeeper_UI_Save()
	local strNewProfileName = GlyphKeeper_SubTextBox:GetText()
	local arraytemp1, arraytemp2, boolFound = GlyphKeeper_GetProfile(strNewProfileName)

	if strNewProfileName == "" then
	elseif boolFound then
		GlyphKeeper_UI_ShowSub("UpdateMode")
	else
		GlyphKeeper_SaveProfile(strNewProfileName)
		GlyphKeeper_UI_HideSub()
	end
end

function GlyphKeeper_UI_Update()
	local strNewProfileName = GlyphKeeper_SubTextBox:GetText()
	
	GlyphKeeper_SaveProfile(strNewProfileName)
	GlyphKeeper_UI_HideSub()
end

function GlyphKeeper_UI_HideSub()
	GlyphKeeper_UI_Sub:Hide()
end

function GlyphKeeper_UI_SubOKClick()
	local strSubLabel = GlyphKeeper_SubLabel:GetText()
	
	if strSubLabel == "Save :" then
		GlyphKeeper_UI_Save()
	elseif strSubLabel == "Update :" then
		GlyphKeeper_UI_Update()
	else
		GlyphKeeper_UI_Delete()
	end
end

function GlyphKeeper_UI_SetSubTextBox()
	local strSubLabel = GlyphKeeper_SubLabel:GetText()

	if strCurrentProfile ~= nil and strSubLabel == "Delete :" then
		GlyphKeeper_SubTextBox:SetText(strCurrentProfile)
	elseif strCurrentProfile ~= nil and strSubLabel == "Save :" then
		GlyphKeeper_SubTextBox:SetText(strCurrentProfile)
	elseif strCurrentProfile == nil then
		GlyphKeeper_SubTextBox:SetText("")
	end
end

function GlyphKeeper_UI_BuildDropDown(self,level)
	local tableSortedProfiles = GlyphKeeper_DeepCopy(tableProfiles)
	local objItem = UIDropDownMenu_CreateInfo()
	
	table.sort(tableSortedProfiles)
	objItem.func = GlyphKeeper_UI_SetProfile;
	for index = 1,#(tableSortedProfiles),1 do
		objItem.text = tableSortedProfiles[index];
		objItem.value = tableSortedProfiles[index];
		if (strCurrentProfile == tableSortedProfiles[index]) then
			objItem.checked = true
		else
			objItem.checked = false
		end
		UIDropDownMenu_AddButton(objItem,level)
	end
end

function GlyphKeeper_UI_SetProfile(newValue)
	strCurrentProfile = newValue.value
	UIDropDownMenu_SetText(GlyphKeeper_Profile_Select, newValue.value)
	UIDropDownMenu_SetSelectedValue(GlyphKeeper_Profile_Select, newValue.value)
end

function GlyphKeeper_UI_CheckProfile(strProfileName)
	local arrayTalents={}
	local arrayGlyphs={}
	local strOutput
	local boolFound

	arrayTalents,arrayGlyphs,boolFound = GlyphKeeper_GetProfile(strProfileName)
	if boolFound then
		strOutput = GlyphKeeper_CompareProfileToActive(strProfileName,arrayTalents,arrayGlyphs)
		GlyphKeeper_CheckList:SetText("Profile : "..strProfileName.."\n\n"..strOutput)
		GlyphKeeper_CheckList_UI:Show()
		GlyphKeeper_Checklist_Toggle_Button:SetText("Show All")
	else
		print("GlyphKeeper - Profile not found : "..strProfileName)
	end

end

function GlyphKeeper_UI_ToggleProfileView()
	local arrayTalents={}
	local arrayGlyphs={}
	local strOutput
	local boolFound
	
	arrayTalents,arrayGlyphs,boolFound = GlyphKeeper_GetProfile(strCurrentProfile)
	if boolFound then
		if GlyphKeeper_Checklist_Toggle_Button:GetText() == "Show Changes" then
			strOutput = GlyphKeeper_CompareProfileToActive(strCurrentProfile,arrayTalents,arrayGlyphs)
			GlyphKeeper_CheckList:SetText("Profile : "..strCurrentProfile.."\n\n"..strOutput)
			GlyphKeeper_Checklist_Toggle_Button:SetText("Show All")
		else
			strOutput = GlyphKeeper_ShowProfileTalentsGlyphs(strCurrentProfile,arrayTalents,arrayGlyphs)
			GlyphKeeper_CheckList:SetText("Profile : "..strCurrentProfile.."\n\n"..strOutput)
			GlyphKeeper_Checklist_Toggle_Button:SetText("Show Changes")
		end
	end
end
----------------------------------------------
--  Logic Functions
----------------------------------------------

function GlyphKeeper_CompareProfileToActive(strProfileName,arrayProfileTalents,arrayProfileGlyphs)
	local boolProfileFound = false;
	local boolActiveFound = false;
	local boolFoundChange = false;
	local arrayActiveTalents = {};
	local arrayActiveGlyphs = {};
	local strOutput = "Talent Changes :\n"

	arrayActiveTalents,arrayActiveGlyphs = GlyphKeeper_GetActiveTalentsGlyphs(false)

	for indexProfile,valueProfile in pairs(arrayProfileTalents) do
		boolProfileFound=false		
		for indexActive,valueActive in pairs(arrayActiveTalents) do
			if ( valueProfile == valueActive ) then
				boolProfileFound=true
			end
		end
		if boolProfileFound == false then
			strOutput = strOutput.."Change Tier "..indexProfile.." to "..valueProfile.."\n"
			boolFoundChange=true
		end
	end
	if boolFoundChange == false then
		strOutput = strOutput.."All Talents match the profile.".."\n"
	end

	boolFoundChange = false

	strOutput = strOutput.."\nGlyph Changes :\n"

	for indexActive,valueActive in pairs(arrayActiveGlyphs) do
		boolActiveFound=false		
		for indexProfile,valueProfile in pairs(arrayProfileGlyphs) do
			if ( valueActive == valueProfile ) then
				boolActiveFound=true
			end
		end
		if boolActiveFound == false then
			strOutput = strOutput.."Remove "..valueActive.."\n"
			boolFoundChange=true
		end
	end

	for indexProfile,valueProfile in pairs(arrayProfileGlyphs) do
		boolProfileFound=false		
		for indexActive,valueActive in pairs(arrayActiveGlyphs) do
			if ( valueProfile == valueActive ) then
				boolProfileFound=true
			end
		end
		if boolProfileFound == false then
			strOutput = strOutput.."Add "..valueProfile.."\n"
			boolFoundChange=true
		end
	end
	if boolFoundChange == false then
		strOutput = strOutput.."All Glyphs match the profile.\n"
	end
	
	return(strOutput)
end

function GlyphKeeper_ShowProfileTalentsGlyphs(strProfileName,arrayTalents,arrayGlyphs)
	local strOutput = "All Talents :\n"
	
	for index,value in pairs(arrayTalents) do
		strOutput = strOutput.." Tier "..(index).." is "..tostring(value).."\n"
	end

	strOutput = strOutput.."\nAll Glyphs :\n"	
	for index,value in pairs(arrayGlyphs) do
		strOutput = strOutput..tostring(value).."\n"
	end
	return strOutput
end

function GlyphKeeper_ShowProfile(strProfileName)
	local arrayTalents={}
	local arrayGlyphs={}
	local strOutput
	local boolFound

	arrayTalents,arrayGlyphs,boolFound = GlyphKeeper_GetProfile(strProfileName)
	if boolFound then
		strOutput = GlyphKeeper_ShowProfileTalentsGlyphs(strProfileName,arrayTalents,arrayGlyphs)
		print(strOutput)
	else
		print("GlyphKeeper - Profile not found : "..strProfileName)
	end
end

function GlyphKeeper_CheckProfile(strProfileName)
	local arrayTalents={}
	local arrayGlyphs={}
	local strOutput
	local boolFound

	arrayTalents,arrayGlyphs,boolFound = GlyphKeeper_GetProfile(strProfileName)
	if boolFound then
		strOutput = GlyphKeeper_CompareProfileToActive(strProfileName,arrayTalents,arrayGlyphs)
		print(strOutput)
	else
		print("GlyphKeeper - Profile not found : "..strProfileName)
	end
end

function GlyphKeeper_DeepCopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[GlyphKeeper_DeepCopy(orig_key)] = GlyphKeeper_DeepCopy(orig_value)
        end
        setmetatable(copy, GlyphKeeper_DeepCopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

----------------------------------------------
--  Data Storage and Retrieval
----------------------------------------------

function GlyphKeeper_GetActiveTalentsGlyphs(boolShowOutput)

	local arrayTalents = {};
	local arrayGlyphs = {};
	
	if boolShowOutput then
		print("GlyphKeeper - Active Talents :")
	end

	for i=1, MAX_NUM_TALENTS do
		local name, _, tier, _, selected = GetTalentInfo(i)
		if selected then
			if boolShowOutput then
				print(" Tier "..tier.." : " ..name)
			end
			tinsert(arrayTalents,name)
		end
	end
	if boolShowOutput then
		print("GlyphKeeper - Active Glyphs :")
	end

	for i = 1, NUM_GLYPH_SLOTS do
		local enabled, glyphType, glyphTooltipIndex, glyphSpellID, icon = GetGlyphSocketInfo(i)
		if ( enabled ) then
			local link = GetGlyphLink(i);-- Retrieves the Glyph's link ("" if no glyph in Socket)
			if ( link ~= "") then
				tinsert(arrayGlyphs,link)
				if boolShowOutput then
					print(" "..link)
				end
			else
				if boolShowOutput then
					print(" Empty Socket")
				end
			end
		else
			if boolShowOutput then
				print (" Unavailable Socket")
			end
		end
	end

	return arrayTalents,arrayGlyphs
end

function GlyphKeeper_GetProfile(strProfileName)
	local arrayTemp = {};
	local arrayTalents = {};
	local arrayGlyphs = {};
	local boolFound = false;
	local indexFound;

	for index,value in pairs(tableProfiles) do
		if ( value == strProfileName ) then
			indexFound = index
			boolFound = true
		end
	end
	
	if boolFound then
		arrayTalents = GlyphKeeper_DeepCopy(tableTalents[indexFound])
		arrayGlyphs = GlyphKeeper_DeepCopy(tableGlyphs[indexFound])
	end

	return arrayTalents,arrayGlyphs,boolFound
end

function GlyphKeeper_SaveProfile(strProfileName)
	local arrayTalents = {};
	local arrayGlyphs = {};

	arrayTalents, arrayGlyphs = GlyphKeeper_GetActiveTalentsGlyphs(false)

	for index,value in ipairs(tableProfiles) do
		if ( value == strProfileName ) then
			print("GlyphKeeper - Replacing Profile : "..strProfileName)
			tremove(tableProfiles,index)
			tremove(tableGlyphs,index)
			tremove(tableTalents,index)
		end
	end

	tinsert(tableProfiles,strProfileName)
	tinsert(tableGlyphs,arrayGlyphs)
	tinsert(tableTalents,arrayTalents)
	print("GlyphKeeper - Total Glyph Profiles : "..#(tableProfiles))
	
end

function GlyphKeeper_DeleteProfile(strProfileName)
	local boolFound = false
	for index,value in ipairs(tableProfiles) do
		if ( value == strProfileName ) then
			boolFound = true
			tremove(tableProfiles,index)
			tremove(tableGlyphs,index)
			tremove(tableTalents,index)
			print("GlyphKeeper - Deleting Profile : "..strProfileName)
		end
	end
	if boolFound == false then
		print("GlyphKeeper - Profile not found : "..strProfileName)
		
	end
end

function GlyphKeeper_ListProfiles()
	for index,value in pairs(tableProfiles) do
		print(" "..value)
	end
end

function GlyphKeeper_CheckDataInteg()
	local numProfileCount = #(tableProfiles)
	local numTalentCount = #(tableTalents) 
	local numGlyphCount = #(tableGlyphs)
	
	if numProfileCount ~= numTalentCount or numProfileCount ~= numGlyphCount then
		print("GlyphKeeper - Data Integrity Error!")
		print("GlyphKeeper - Please run /gk reset!")
	end
end

function GlyphKeeper_ClearProfiles()
	wipe(tableProfiles)
	wipe(tableGlyphs)
	wipe(tableTalents)
end

----------------------------------------------
-- CLI and Event Functions
----------------------------------------------

function GlyphKeeper_CLI(msg, editBox)
	local command, rest = msg:match("^(%S*)%s*(.-)$")

	if command == "list" then
		print('Glyphkeeper - Listing Profiles:')
		GlyphKeeper_ListProfiles()
		print('Glyphkeeper - End of Profiles')
	elseif command == "check" then
		print('Glyphkeeper - Checking '..rest)
		GlyphKeeper_CheckProfile(rest)
	elseif command == "profile" then
		print('Glyphkeeper - Showing '..rest)
		GlyphKeeper_ShowProfile(rest)
	elseif command == "save" then
	 	print('Glyphkeeper - Saving '..rest)
		GlyphKeeper_SaveProfile(rest)
	elseif command == "delete" then
	 	print('Glyphkeeper - Deleting '..rest)
		GlyphKeeper_DeleteProfile(rest)
	elseif command == "reset" then
		if rest == "yes" then
			print('Glyphkeeper - Clearing Profiles.')
			GlyphKeeper_ClearProfiles()
		else
			print("Glyphkeeper - This command will delete all profiles. Please confirm by typing : /gk reset yes")
		end
	elseif command == "show" then
		GlyphKeeper_UI_ShowToggle()
	else
		print("Glyphkeeper commands")
		print(" /gk list - Lists all saved profiles")
		print(" /gk check profilename - Shows the glyphs and talents that are different than the profilename")
		print(" /gk profile profilename - Shows the glyphs and talents in the profilename")
		print(" /gk save profilename - Saves current active glyphs and talents into profilename")
		print(" /gk delete profilename - Deletes the profilename")
		print(" /gk reset - Clears all profiles. Use only if Glyphkeeper won't work.")
		print(" /gk show - Shows UI")
	end
end

function GlyphKeeper_OnEvent(self, event, ...)

	if ( event == "VARIABLES_LOADED" ) then
		if tableProfiles == nil then tableProfiles = {} end;
		if tableGlyphs == nil then tableGlyphs = {} end;
		if tableTalents == nil then tableTalents = {} end;
		
		print("GlyphKeeper 2.4 - Type /gk to use.")
		GlyphKeeper_CheckDataInteg()
	elseif ( event == "READY_CHECK" ) then
		print("GlyphKeeper - Have you checked your glyphs and talents?")
--	elseif ( event == "PLAYER_TALENT_UPDATE" or event == "GLYPH_ADDED") then
--		print("GlyphKeeper - You've changed a talent or glyph.  Click the GK button to save a profile.")
	end
end

local strCurrentProfile
SLASH_GLYPHKEEPER1 = "/gk"
SlashCmdList["GLYPHKEEPER"] = function(msg, editBox)
	GlyphKeeper_CLI(msg, editBox)
end

GlyphKeeper = CreateFrame("Frame")
GlyphKeeper:RegisterEvent("VARIABLES_LOADED")
GlyphKeeper:RegisterEvent("PLAYER_TALENT_UPDATE")
GlyphKeeper:RegisterEvent("GLYPH_ADDED")
GlyphKeeper:RegisterEvent("READY_CHECK")
GlyphKeeper:SetScript("OnEvent", GlyphKeeper_OnEvent)
